@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: gssapi.h,v 1.10 1997/08/19 10:10:42 assar Exp $ */
+/* $Id: gssapi.h,v 1.14 1999/12/02 17:05:03 joda Exp $ */
 
 #ifndef GSSAPI_H_
 #define GSSAPI_H_
@@ -52,7 +47,7 @@
 
 #include <sys/types.h>
 
-#include <bits.h>
+#include <krb5-types.h>
 
 /*
  * Now define the three implementation-dependent types.
@@ -77,6 +72,7 @@ typedef struct gss_ctx_id_t_desc_struct {
   gss_name_t source, target;
   OM_uint32 flags;
   enum { LOCAL = 1, OPEN = 2} more_flags;
+  struct krb5_ticket *ticket;
 } gss_ctx_id_t_desc;
 
 typedef gss_ctx_id_t_desc *gss_ctx_id_t;
@@ -173,6 +169,7 @@ typedef OM_uint32 gss_qop_t;
 #define GSS_C_AF_DSS        18
 #define GSS_C_AF_OSI        19
 #define GSS_C_AF_X25        21
+#define GSS_C_AF_INET6	    24
 
 #define GSS_C_AF_NULLADDR   255
 
