@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -37,13 +32,15 @@
  */
 
 #include "krb5_locl.h"
+#ifdef HAVE_RES_SEARCH
 #define USE_RESOLVER
+#endif
 #ifdef HAVE_ARPA_NAMESER_H
 #include <arpa/nameser.h>
 #endif
 #include "resolve.h"
 
-RCSID("$Id: principal.c,v 1.45 1999/04/24 16:42:23 joda Exp $");
+RCSID("$Id: principal.c,v 1.54 1999/12/16 11:48:25 assar Exp $");
 
 #define princ_num_comp(P) ((P)->name.name_string.len)
 #define princ_type(P) ((P)->name.name_type)
@@ -66,7 +63,7 @@ krb5_parse_name(krb5_context context,
 		const char *name,
 		krb5_principal *principal)
 {
-
+    krb5_error_code ret;
     general_string *comp;
     general_string realm;
     int ncomp;
@@ -91,9 +88,15 @@ krb5_parse_name(krb5_context context,
 	    ncomp++;
     }
     comp = calloc(ncomp, sizeof(*comp));
+    if (comp == NULL)
+	return ENOMEM;
   
     n = 0;
     start = q = p = s = strdup(name);
+    if (start == NULL) {
+	free (comp);
+	return ENOMEM;
+    }
     while(*p){
 	c = *p++;
 	if(c == '\\'){
@@ -108,15 +111,14 @@ krb5_parse_name(krb5_context context,
 		c = '\0';
 	}else if(c == '/' || c == '@'){
 	    if(got_realm){
-	    exit:
-		while(n>0){
-		    free(comp[--n]);
-		}
-		free(comp);
-		free(s);
-		return KRB5_PARSE_MALFORMED;
+		ret = KRB5_PARSE_MALFORMED;
+		goto exit;
 	    }else{
 		comp[n] = malloc(q - start + 1);
+		if (comp[n] == NULL) {
+		    ret = ENOMEM;
+		    goto exit;
+		}
 		strncpy(comp[n], start, q - start);
 		comp[n][q - start] = 0;
 		n++;
@@ -126,29 +128,52 @@ krb5_parse_name(krb5_context context,
 	    start = q;
 	    continue;
 	}
-	if(got_realm && (c == ':' || c == '/' || c == '\0'))
+	if(got_realm && (c == ':' || c == '/' || c == '\0')) {
+	    ret = KRB5_PARSE_MALFORMED;
 	    goto exit;
+	}
 	*q++ = c;
     }
     if(got_realm){
 	realm = malloc(q - start + 1);
+	if (realm == NULL) {
+	    ret = ENOMEM;
+	    goto exit;
+	}
 	strncpy(realm, start, q - start);
 	realm[q - start] = 0;
     }else{
-	krb5_get_default_realm (context, &realm);
+	ret = krb5_get_default_realm (context, &realm);
+	if (ret)
+	    goto exit;
 
 	comp[n] = malloc(q - start + 1);
+	if (comp[n] == NULL) {
+	    ret = ENOMEM;
+	    goto exit;
+	}
 	strncpy(comp[n], start, q - start);
 	comp[n][q - start] = 0;
 	n++;
     }
     *principal = malloc(sizeof(**principal));
+    if (*principal == NULL) {
+	ret = ENOMEM;
+	goto exit;
+    }
     (*principal)->name.name_type = KRB5_NT_PRINCIPAL;
     (*principal)->name.name_string.val = comp;
     princ_num_comp(*principal) = n;
     (*principal)->realm = realm;
     free(s);
     return 0;
+exit:
+    while(n>0){
+	free(comp[--n]);
+    }
+    free(comp);
+    free(s);
+    return ret;
 }
 
 static const char quotable_chars[] = "\n\t\b\\/@";
@@ -252,7 +277,7 @@ unparse_name(krb5_context context,
 	len++;
     }
     *name = malloc(len);
-    if(*name == NULL)
+    if(len != 0 && *name == NULL)
 	return ENOMEM;
     ret = unparse_name_fixed(context, principal, *name, len, short_flag);
     if(ret)
@@ -401,7 +426,7 @@ krb5_make_principal(krb5_context context,
     krb5_error_code ret;
     krb5_realm r = NULL;
     va_list ap;
-    if(realm == NULL){
+    if(realm == NULL) {
 	ret = krb5_get_default_realm(context, &r);
 	if(ret)
 	    return ret;
@@ -650,6 +675,8 @@ krb5_425_conv_principal_ext(krb5_context context,
 	return HEIM_ERR_V4_PRINC_NO_CONV;
     }
 	
+    if (*p == '.')
+	++p;
     snprintf(host, sizeof(host), "%s.%s", instance, p);
     ret = krb5_make_principal(context, &pr, realm, name, host, NULL);
     if(func == NULL || (*func)(context, pr)){
@@ -839,7 +866,8 @@ krb5_sname_to_principal (krb5_context context,
     krb5_error_code ret;
     char localhost[128];
     char **realms, *host = NULL;
-    
+    struct hostent *hp = NULL;
+	
     if(type != KRB5_NT_SRV_HST && type != KRB5_NT_UNKNOWN)
 	return KRB5_SNAME_UNSUPP_NAMETYPE;
     if(hostname == NULL){
@@ -848,27 +876,41 @@ krb5_sname_to_principal (krb5_context context,
     }
     if(sname == NULL)
 	sname = "host";
-    if(type == KRB5_NT_SRV_HST){
-	struct hostent *hp;
-	hp = roken_gethostbyname(hostname);
-	if(hp != NULL)
-	    hostname = hp->h_name;
-    }
-    if(type == KRB5_NT_SRV_HST){
-	host = strdup(hostname);
-	if(host == NULL){
-	    return ENOMEM;
+    if(type == KRB5_NT_SRV_HST) {
+	int error;
+	struct addrinfo hints, *res;
+	char *host;
+
+	memset (&hints, 0, sizeof(hints));
+	hints.ai_flags = AI_CANONNAME;
+
+	error = getaddrinfo (hostname, NULL, &hints, &res);
+	if (error == 0) {
+	    host = strdup (res->ai_canonname);
+	    freeaddrinfo (res);
+	} else if (res->ai_canonname == NULL) {
+	    freeaddrinfo (res);
+	    host = strdup (hostname);
+	} else {
+	    host = strdup (hostname);
 	}
+	if (host == NULL)
+	    return ENOMEM;
 	strlwr(host);
 	hostname = host;
     }
     ret = krb5_get_host_realm(context, hostname, &realms);
-    if(ret)
+    if(ret) {
+	if (hp != NULL)
+	    freehostent (hp);
 	return ret;
+    }
     ret = krb5_make_principal(context, ret_princ, realms[0], sname,
 			      hostname, NULL);
     if(host)
 	free(host);
+    if (hp)
+	freehostent (hp);
     krb5_free_host_realm(context, realms);
     return ret;
 }
